// ignore_for_file: public_member_api_docs

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:healthrian_ble_support_for_wearecg12/'
    'healthrian_ble_support_for_wearecg12.dart';
import 'package:healthrian_visualization_support/'
    'healthrian_visualization_support.dart';

part 'ecg_waerecg_quarter.g.dart';

@CopyWith()
class EcgWearECGQuarter extends StatelessWidget {
  const EcgWearECGQuarter({
    super.key,
    this.onLoadingWidget,
    this.streams,
    this.height = 50,
    this.color = Colors.black,
    this.measureTime = 5,
    this.wrapperWidget,
    this.pivotColor = Colors.red,
  });

  final int measureTime;
  final Color color;
  final Streams? streams;
  final double height;
  final Widget? onLoadingWidget;
  final Color pivotColor;

  final Widget Function(List<Widget>)? wrapperWidget;

  @override
  Widget build(BuildContext context) {
    return Row(
      children: [
        Observer(
          key: UniqueKey(),
          builder: (context) {
            return SizedBox(
              width: MediaQuery.of(context).size.width / 4,
              height: height * 12,
              child: StreamBuilder(
                stream: BLEManager().streams.bufferedPlotRTStreamQ1,
                builder: (context, snapshot) {
                  if (snapshot.connectionState != ConnectionState.active) {
                    return onLoadingWidget ?? const SizedBox();
                  }

                  final list = snapshot.data!.toList();
                  final pivot = snapshot.data!.pivot;

                  final children = [
                    for (final channel in list)
                      SizedBox(
                        width: MediaQuery.of(context).size.width / 4,
                        height: height,
                        child: ChannelWithPivot(
                          ecgSequence: channel,
                          color: color,
                          measureTime: measureTime,
                          pivot: pivot,
                          pivotColor: pivotColor,
                          division: 4,
                        ),
                      ),
                  ];

                  if (wrapperWidget == null) {
                    return Column(
                      children: children,
                    );
                  }
                  return wrapperWidget!(children);
                },
              ),
            );
          },
        ),

        Observer(
          key: UniqueKey(),
          builder: (context) {
            return SizedBox(
              width: MediaQuery.of(context).size.width / 4,
              height: height * 12,
              child: StreamBuilder(
                stream: BLEManager().streams.bufferedPlotRTStreamQ2,
                builder: (context, snapshot) {
                  if (snapshot.connectionState != ConnectionState.active) {
                    return onLoadingWidget ?? const SizedBox();
                  }

                  final list = snapshot.data!.toList();
                  final pivot = snapshot.data!.pivot;

                  final children = [
                    for (final channel in list)
                      SizedBox(
                        width: MediaQuery.of(context).size.width,
                        height: height,
                        child: ChannelWithPivot(
                          ecgSequence: channel,
                          color: color,
                          measureTime: measureTime,
                          pivot: pivot,
                          pivotColor: pivotColor,
                          division: 4,
                          step: 1,
                        ),
                      ),
                  ];

                  if (wrapperWidget == null) {
                    return Column(
                      children: children,
                    );
                  }
                  return wrapperWidget!(children);
                },
              ),
            );
          },
        ),

        Observer(
          key: UniqueKey(),
          builder: (context) {
            return SizedBox(
              width: MediaQuery.of(context).size.width / 4,
              height: height * 12,
              child: StreamBuilder(
                stream: BLEManager().streams.bufferedPlotRTStreamQ3,
                builder: (context, snapshot) {
                  if (snapshot.connectionState != ConnectionState.active) {
                    return onLoadingWidget ?? const SizedBox();
                  }

                  final list = snapshot.data!.toList();
                  final pivot = snapshot.data!.pivot;

                  final children = [
                    for (final channel in list)
                      SizedBox(
                        width: MediaQuery.of(context).size.width / 4,
                        height: height,
                        child: ChannelWithPivot(
                          ecgSequence: channel,
                          color: color,
                          measureTime: measureTime,
                          pivot: pivot,
                          pivotColor: pivotColor,
                          division: 4,
                          step: 2,
                        ),
                      ),
                  ];

                  if (wrapperWidget == null) {
                    return Column(
                      children: children,
                    );
                  }
                  return wrapperWidget!(children);
                },
              ),
            );
          },
        ),

        Observer(
          key: UniqueKey(),
          builder: (context) {
            return SizedBox(
              width: MediaQuery.of(context).size.width / 4,
              height: height * 12,
              child: StreamBuilder(
                stream: BLEManager().streams.bufferedPlotRTStreamQ4,
                builder: (context, snapshot) {
                  if (snapshot.connectionState != ConnectionState.active) {
                    return onLoadingWidget ?? const SizedBox();
                  }

                  final list = snapshot.data!.toList();
                  final pivot = snapshot.data!.pivot;

                  final children = [
                    for (final channel in list)
                      SizedBox(
                        width: MediaQuery.of(context).size.width / 4,
                        height: height,
                        child: ChannelWithPivot(
                          ecgSequence: channel,
                          color: color,
                          measureTime: measureTime,
                          pivot: pivot,
                          pivotColor: pivotColor,
                          division: 4,
                          step: 3,
                        ),
                      ),
                  ];

                  if (wrapperWidget == null) {
                    return Column(
                      children: children,
                    );
                  }
                  return wrapperWidget!(children);
                },
              ),
            );
          },
        ),

        // for (final stream in [
        //   BLEManager().streams.bufferedPlotRTStreamQ1,
        //   BLEManager().streams.bufferedPlotRTStreamQ2,
        //   BLEManager().streams.bufferedPlotRTStreamQ3,
        //   BLEManager().streams.bufferedPlotRTStreamQ4,
        // ])
        //   SizedBox(
        //     width: MediaQuery.of(context).size.width / 4,
        //     height: height,
        //     child: StreamBuilder(
        //       stream: BLEManager().streams.bufferedPlotRTStreamQ1,
        //       builder: (context, snapshot) {
        //         if (!snapshot.hasData) {
        //           return onLoadingWidget ?? const SizedBox();
        //         }
        //
        //         final list = snapshot.data!.toList();
        //         final pivot = snapshot.data!.pivot;
        //
        //         final children = [
        //           for (final channel in list)
        //             SizedBox(
        //               width: MediaQuery.of(context).size.width,
        //               height: height,
        //               child: ChannelWithPivot(
        //                 ecgSequence: channel,
        //                 color: color,
        //                 measureTime: 1,
        //                 pivot: pivot,
        //                 pivotColor: pivotColor,
        //               ),
        //             ),
        //         ];
        //
        //         if (wrapperWidget == null) {
        //           return Column(
        //             children: children,
        //           );
        //         }
        //         return wrapperWidget!(children);
        //       },
        //     ),
        //   )
      ],
    );
  }
}
